<?php

use Imoje\Payment\Api;
use Imoje\Payment\CartData;
use Imoje\Payment\Invoice;
use Imoje\Payment\Notification;
use Imoje\Payment\Util;

/**
 * Class Helper
 */
class Helper
{

	/**
	 * @return string
	 */
	public static function get_version()
	{
		$wc = WC();

		return get_file_data(dirname(dirname(__FILE__)) . '/woocommerce-imoje.php', ['Version'], 'plugin')[0] . ';woocommerce_' . $wc->version;
	}

	/**
	 * @param WC_Order $order
	 * @param bool     $isEnable
	 * @param bool     $isApi
	 *
	 * @return array|string
	 */
	public static function get_invoice($order, $isEnable, $isApi)
	{

		if($isEnable) {

			$cart = self::get_cart($order);

			$invoice = new Invoice();

			if(isset($cart['items']) && $cart['items']) {

				foreach($cart['items'] as $item) {

					$tax = null;
					try {
						$tax = constant('\Imoje\Payment\Invoice::TAX_' . $item['vat']);
					} catch(Exception $e) {
					}

					if(!$tax) {
						return [];
					}

					$invoice->addItem($item['name'],
						$item['id'],
						$item['quantity'],
						constant('\Imoje\Payment\Invoice::TAX_' . $item['vat']),
						$item['amount']
					);
				}

				$invoice->setBuyer(Invoice::BUYER_PERSON,
					$order->get_billing_email(),
					$cart['address']['billing']['name'],
					$cart['address']['billing']['street'],
					$cart['address']['billing']['city'],
					$cart['address']['billing']['postalCode'],
					$cart['address']['billing']['country']);

				return $invoice->prepare($isApi);
			}
		}

		return [];
	}

	/**
	 * @param WC_Order $order
	 *
	 * @return array
	 */
	public static function get_cart($order)
	{

		$cart_data = new CartData();

		$cart_data->setCreatedAt(strtotime($order->get_date_created()));
		$cart_data->setAmount(Util::convertAmountToFractional($order->get_total()));

		foreach($order->get_items() as $item) {

			$product = $item->get_product();

			$tax_rate_product = current(WC_Tax::get_rates($product->get_tax_class()));

			$rate = 0;
			if($tax_rate_product && isset($tax_rate_product['rate'])) {
				$rate = (float) $tax_rate_product['rate'];
			}
			// endregion

			$cart_data->addItem(
				$product->get_id(),
				$rate,
				$item->get_name(),
				Util::convertAmountToFractional($item->get_total()),
				$item->get_quantity()
			);
		}

		$phone = $order->get_billing_phone();

		$cart_data->setAddressBilling(
			$order->get_billing_city(),
			$order->get_billing_first_name() . ' ' . $order->get_billing_last_name(),
			$phone
				?: '',
			$order->get_billing_address_1() . ' ' . $order->get_billing_address_2(),
			$order->get_billing_country(),
			$order->get_billing_postcode()
		);

		if($order->get_total_discount(false) > 0) {
			$cart_data->setDiscount(
				0,
				'discount',
				Util::convertAmountToFractional($order->get_total_discount(false))
			);
		}

		$shipping_data = current($order->get_items('shipping'));

		if(!$shipping_data) {
			$cart_data->setAddressDelivery(
				$order->get_billing_city(),
				$order->get_billing_first_name() . ' ' . $order->get_billing_last_name(),
				$phone
					?: '',
				$order->get_billing_address_1() . ' ' . $order->get_billing_address_2(),
				$order->get_billing_country(),
				$order->get_billing_postcode()
			);
			$cart_data->setShipping(
				0,
				'noshipping',
				0
			);

			return $cart_data->prepareCartDataArray();
		}

		$shipping_data = $shipping_data->get_data();
		$id_shipping_tax = null;
		foreach($order->get_items('shipping') as $item_id => $item_tax) {
			$shipping_data = json_decode($item_tax, true);
			$id_shipping_tax = current(array_keys($shipping_data['taxes']['total']));
		}

		$cart_data->setShipping($id_shipping_tax
			? (float) WC_Tax::_get_tax_rate($id_shipping_tax)['tax_rate']
			: 0,
			$shipping_data['name'],
			Util::convertAmountToFractional($shipping_data['total'] + $shipping_data['total_tax']));

		$cart_data->setAddressDelivery(
			$order->get_shipping_city(),
			$order->get_shipping_first_name() . ' ' . $order->get_shipping_last_name(),
			$phone
				?: '',
			$order->get_shipping_address_1() . ' ' . $order->get_shipping_address_2(),
			$order->get_shipping_country(),
			$order->get_shipping_postcode()
		);

		return $cart_data->prepareCartDataArray();
	}

	/**
	 * @param string $service_id
	 * @param string $service_key
	 * @param string $debug_mode
	 *
	 * @throws Exception
	 */
	public static function check_notification($service_id, $service_key, $debug_mode)
	{
		$notification = new Notification(
			$service_id,
			$service_key
		);

		// it can be order data or notification code - depends on verification notification
		$result_check_request_notification = $notification->checkRequest();
		$is_debug_mode = $debug_mode === 'yes';

		if(is_int($result_check_request_notification)) {
			echo Notification::formatResponse(Notification::NS_ERROR, $result_check_request_notification, $is_debug_mode);
			exit();
		}

		if(!($order = wc_get_order($result_check_request_notification['transaction']['orderId']))) {

			echo Notification::formatResponse(Notification::NS_ERROR, Notification::NC_ORDER_NOT_FOUND, $is_debug_mode);
			exit();
		}

		if($order->get_status() === 'completed' || $order->get_status() === 'processing') {

			echo Notification::formatResponse(Notification::NS_ERROR, Notification::NC_INVALID_ORDER_STATUS, $is_debug_mode);
			exit();
		}

		if(!Notification::checkRequestAmount($result_check_request_notification, Util::convertAmountToFractional($order->data['total']), $order->data['currency'])) {

			echo Notification::formatResponse(Notification::NS_ERROR, Notification::NC_AMOUNT_NOT_MATCH, $is_debug_mode);
			exit();
		}

		$transactionStatuses = Util::getTransactionStatuses();

		if(!isset($transactionStatuses[$result_check_request_notification['transaction']['status']])) {
			echo Notification::formatResponse(Notification::NS_ERROR, Notification::NC_UNHANDLED_STATUS, $is_debug_mode);
			exit;
		}

		switch($result_check_request_notification['transaction']['status']) {
			case 'settled':

				$order->update_status($order->needs_processing()
					? 'processing'
					: 'completed');

				$order->update_meta_data('imoje_transaction_uuid', $result_check_request_notification['transaction']['id']);
				$order->save_meta_data();

				wc_reduce_stock_levels($order->get_id());

				$order->add_order_note(__('Transaction reference', 'imoje') . ': ' . $result_check_request_notification['transaction']['id']);
				echo Notification::formatResponse(Notification::NS_OK);
				exit;
			case 'rejected':
				$order->update_status('failed');
				$order->add_order_note(__('Transaction reference', 'imoje') . ': ' . $result_check_request_notification['transaction']['id']);
				echo Notification::formatResponse(Notification::NS_OK);
				exit;
			default:
				echo Notification::formatResponse(Notification::NS_OK, Notification::NC_UNHANDLED_STATUS, $is_debug_mode);
				exit;
		}
	}

	/**
	 * @param int    $order_id
	 * @param string $authorization_token
	 * @param string $merchant_id
	 * @param string $service_id
	 * @param float  $amount
	 *
	 * @return bool
	 * @throws Exception
	 */
	public static function process_refund($order_id, $authorization_token, $merchant_id, $service_id, $amount)
	{

		$order = wc_get_order($order_id);

		$api = new Api($authorization_token, $merchant_id, $service_id);

		$refund = $api->createRefund(
			$api->prepareRefundData(
				Util::convertAmountToFractional($amount)
			),
			$order->get_meta('imoje_transaction_uuid')
		);

		if($refund['success']) {

			if($refund['body']['transaction']['status'] !== 'settled') {
				return false;
			}

			$order->add_order_note(
				__('Refund for amount:', 'imoje')
				. ' '
				. $amount
				. ' '
				. __('with UUID:', 'imoje')
				. ' '
				. $refund['body']['transaction']['id']
				. ' '
				. __('has been created.', 'imoje'));

			return true;
		}

		if(isset($refund['data']['body']) && $refund['data']['body']) {
			$refund_error = __('Refund error: ', 'imoje') . ' ' . $refund['data']['body'];
		} else {
			$refund_error = __('Refund error.', 'imoje');
		}

		$order->add_order_note(
			$refund_error
		);

		throw new Exception($refund_error);
	}
}
