<?php
/*
Plugin Name: WooCommerce imoje
Plugin URI: https://imoje.pl
Description: Add payment via imoje to WooCommerce
Version: 3.2.3
Author: imoje <kontakt.tech@imoje.pl>
Author URI: https://imoje.pl
Text Domain: imoje
*/

use Imoje\Payment\Api;
use Imoje\Payment\Util;

add_action('plugins_loaded', 'init_woocommerce_gateway_imoje', 0);

/**
 * Init function that runs after plugin install.
 */
function init_woocommerce_gateway_imoje()
{
	@include_once __DIR__ . "/includes/libs/payment-core/vendor/autoload.php";
	@include_once __DIR__ . "/includes/libs/Payment-core/vendor/autoload.php";

	if(!class_exists('WC_Payment_Gateway')) {
		return;
	}

	define('WOOCOMMERCE_IMOJE_PLUGIN_DIR', plugin_dir_path(__FILE__));
	define('WOOCOMMERCE_IMOJE_PLUGIN_URL', plugin_dir_url(__FILE__));
	load_plugin_textdomain('imoje', false, dirname(plugin_basename(__FILE__)) . '/langs/');

	require_once('includes/class.WCGatewayImojeBlik.php');
	require_once('includes/class.WCGatewayImoje.php');
	require_once('includes/class.WCGatewayImojePaylater.php');
	require_once('includes/class.WCGatewayImojeCards.php');
	require_once('includes/class.WCGatewayImojePbl.php');

	add_filter('woocommerce_payment_gateways', 'add_imoje_gateways');
}

/**
 * @param $methods
 *
 * @return array
 */
function add_imoje_gateways($methods)
{
	$methods[] = 'WC_Gateway_ImojeBlik';
	$methods[] = 'WC_Gateway_Imoje';
	$methods[] = 'WC_Gateway_ImojePaylater';
	$methods[] = 'WC_Gateway_ImojeCards';
	$methods[] = 'WC_Gateway_ImojePbl';

	return $methods;
}

// region utils

// region ajax check transaction
add_action('wp_ajax_imoje_check_transaction', 'check_transaction_ajax');
add_action('wp_ajax_nopriv_imoje_check_transaction', 'check_transaction_ajax');

function check_transaction_ajax()
{

	if(!isset($_REQUEST['transaction_uuid']) || !$_REQUEST['transaction_uuid']
		|| !isset($_REQUEST['method']) || !$_REQUEST['method']
	) {

		wp_send_json_error();
	}

	$imoje_api = get_api_instance($_REQUEST['method']);

	if(!$imoje_api) {
		wp_send_json_error();
	}

	$transaction = $imoje_api->getTransaction($_REQUEST['transaction_uuid']);

	if($transaction['success']) {
		wp_send_json_success(prepare_success_response($transaction));
	}

	wp_send_json_error();
}

// endregion

// region create transaction
add_action('wp_ajax_imoje_create_transaction', 'create_transaction_ajax');
add_action('wp_ajax_nopriv_imoje_create_transaction', 'create_transaction_ajax');

function create_transaction_ajax()
{

	if((!isset($_REQUEST['method']) && !$_REQUEST['method'])
		|| (!isset($_REQUEST['order_id']) && !$_REQUEST['order_id'])) {
		wp_send_json_error();
	}

	$blik_code = null;

	$is_code = isset($_REQUEST['code']) && $_REQUEST['code'];

	// region blik
	if($is_code
		&& isset($_REQUEST['order_id']) && $_REQUEST['order_id']) {

		if(!is_numeric($_REQUEST['code'])
			|| strlen($_REQUEST['code']) != 6) {

			wp_send_json_error();
		}

		$blik_code = $_REQUEST['code'];
	}

	$order = wc_get_order($_REQUEST['order_id']);

	if(!$order) {
		wp_send_json_error();
	}

	$imoje_api = get_api_instance($_REQUEST['method']);

	if(!$imoje_api) {
		wp_send_json_error();
	}

	$payment_method = '';
	$payment_method_code = '';
	$transaction_body = '';
	$transaction_url = '';

	if($blik_code) {
		$payment_method = Util::getPaymentMethod('blik');
		$payment_method_code = Util::getPaymentMethodCode('blik');

		$is_remember_blik_code = isset($_REQUEST['remember_blik_code']) && $_REQUEST['remember_blik_code'];

		if($is_remember_blik_code) {
			$payment_method_code = Util::getPaymentMethodCode('blik_oneclick');
		}

		if($is_remember_blik_code && $is_code
			&& isset($_REQUEST['profile_id']) && $_REQUEST['profile_id']) {

			$transaction_body = $imoje_api->prepareBlikOneclickData(
				$_REQUEST['profile_id'],
				$order->get_total(),
				$order->get_currency(),
				$_REQUEST['order_id'],
				$_REQUEST['order_id'],
				$_SERVER['REMOTE_ADDR'],
				'',
				$blik_code
			);
			$transaction_url = $imoje_api->getDebitBlikProfileUrl();
		}
	}

	$cid = '';
	if(is_user_logged_in()) {
		$user_id = get_current_user_id();
		$user = get_userdata($user_id);
		$cid = Util::getCid($user_id . $user->user_email);
	}

	if(!$transaction_body) {
		include_once 'includes/Helper.php';

		$options = get_options($_REQUEST['method']);
		$transaction_body = $imoje_api->prepareData(
			$order->get_total(),
			$order->get_currency(),
			$order->get_id(),
			$payment_method,
			$payment_method_code,
			$order->get_checkout_order_received_url(),
			$order->get_checkout_order_received_url(),
			$order->get_billing_first_name(),
			$order->get_billing_last_name(),
			$order->get_billing_email(),
			Api::TRANSACTION_TYPE_SALE,
			'',
			$blik_code,
			'',
			$cid,
			Helper::get_invoice($order, $options['ing_ksiegowosc'] === 'yes', true)
		);
	}

	$transaction = $imoje_api->createTransaction($transaction_body, $transaction_url);

	if($transaction['success']) {

		wp_send_json_success(prepare_success_response($transaction));
	}

	if(isset($transaction['data']['body']) && $transaction['data']['body']) {
		error_log(__('Transaction could not be initialized, error: '), 'imoje')
		. $transaction['data']['body'];
	}

	wp_send_json_error();
}

// endregion

// region deactivate alias
add_action('wp_ajax_imoje_deactivate_alias', 'deactivate_alias');
add_action('wp_ajax_nopriv_imoje_deactivate_alias', 'deactivate_alias');

function deactivate_alias()
{

	if(
		(!isset($_REQUEST['method']) && !$_REQUEST['method'])
		|| (!isset($_REQUEST['profile_id']) && !$_REQUEST['profile_id'])
		|| (!isset($_REQUEST['blik_key']) && !$_REQUEST['blik_key'])
	) {
		wp_send_json_error();
	}

	$imoje_api = get_api_instance($_REQUEST['method']);

	if(!$imoje_api) {
		wp_send_json_error();
	}

	$deactivate = $imoje_api->createTransaction(
		json_encode([
				'blikProfileId' => $_REQUEST['profile_id'],
				'blikKey'       => $_REQUEST['blik_key'],
			]
		),
		$imoje_api->getDeactivateBlikProfileUrl()
	);

	if($deactivate['success']) {
		wp_send_json_success();
	}

	wp_send_json_error();
}

// endregion

// region debit alias
add_action('wp_ajax_imoje_debit_alias', 'debit_alias');
add_action('wp_ajax_nopriv_imoje_debit_alias', 'debit_alias');

function debit_alias()
{

	if(
		(!isset($_REQUEST['method']) && !$_REQUEST['method'])
		|| (!isset($_REQUEST['order_id']) && !$_REQUEST['order_id'])
		|| (!isset($_REQUEST['profile_id']) && !$_REQUEST['profile_id'])
		|| (!isset($_REQUEST['blik_key']) && !$_REQUEST['blik_key'])
	) {
		wp_send_json_error();
	}

	$imoje_api = get_api_instance($_REQUEST['method']);

	if(!$imoje_api) {
		wp_send_json_error();
	}

	$order = wc_get_order($_REQUEST['order_id']);

	if(!$order) {
		wp_send_json_error();
	}

	$transaction = $imoje_api->createTransaction(
		$imoje_api->prepareBlikOneclickData(
			$_REQUEST['profile_id'],
			$order->get_total(),
			$order->get_currency(),
			$_REQUEST['order_id'],
			$_REQUEST['order_id'],
			$_SERVER['REMOTE_ADDR'],
			$_REQUEST['blik_key']
		),
		$imoje_api->getDebitBlikProfileUrl()
	);

	if($transaction['success']) {
		wp_send_json_success(prepare_success_response($transaction));
	}

	wp_send_json_error();
}

// endregion

/**
 * @param $method
 *
 * @return false|Api
 */
function get_api_instance($method)
{

	$options = get_options($method);

	if(!$options
		|| !$options['authorization_token']
		|| !$options['merchant_id']
		|| !$options['service_id']
	) {
		return false;
	}

	return new Api(
		$options['authorization_token'],
		$options['merchant_id'],
		$options['service_id'],
		$options['sandbox'] === 'yes'
			? Util::ENVIRONMENT_SANDBOX
			: Util::ENVIRONMENT_PRODUCTION
	);
}

/**
 * @param array $transaction
 *
 * @return array
 */
function prepare_success_response($transaction)
{

	$array = [
		'transaction' => [
			'status' => $transaction['body']['transaction']['status'],
			'id'     => $transaction['body']['transaction']['id'],
		],
	];

	if(isset($transaction['body']['transaction']['statusCode']) && $transaction['body']['transaction']['statusCode']) {
		$array['error'] = get_error_message($transaction['body']['transaction']['statusCode']);
	}

	if(isset($transaction['body']['code']) && $transaction['body']['code']) {
		$array['code'] = $transaction['body']['code'];
	}

	if(isset($transaction['body']['newParamAlias']) && $transaction['body']['code']) {
		$array['newParamAlias'] = $transaction['body']['newParamAlias'];
	}

	return $array;
}

/**
 * @return array
 */
function get_options($method)
{

	return get_option('woocommerce_' . $method . '_settings');
}

// region blik

/**
 * @param string $code
 *
 * @return string
 */
function get_error_message($code)
{

	$tEnterT6 = __('Insert BLIK code.', 'imoje');
	$tTryAgain = __('Please try again.', 'imoje');

	switch($code) {
		case 'BLK-ERROR-210000':
			return __('Payment failed.', 'imoje') . ' ' . $tTryAgain;
		case 'BLK-ERROR-210001':
			return __('Technical break in your bank. Pay later or use another bank\'s application.', 'imoje');
		case 'BLK-ERROR-210002':
			return __('Alias not found. To proceed the payment you need to pay with BLIK code.', 'imoje');
		case 'BLK-ERROR-210003':
		case 'BLK-ERROR-210004':
			return __('Alias declined. To proceed the payment you need to pay with BLIK code.', 'imoje');
		case 'BLK-ERROR-210005':
			$msg = __('You have entered wrong BLIK code.', 'imoje');
			break;
		case 'BLK-ERROR-210006':
			$msg = __('BLIK code expired.', 'imoje');
			break;
		case 'BLK-ERROR-210007':
		case 'BLK-ERROR-210008':
			$msg = __('Something went wrong with BLIK code.', 'imoje');
			break;
		case 'BLK-ERROR-210009':
			$msg = __('Payment declined at the banking application.', 'imoje');
			break;
		case 'BLK-ERROR-210010':
		case 'BLK-ERROR-210011':
			$msg = __('Payment failed - not confirmed on time in the banking application.', 'imoje');
			break;
		case 'BLK-ERROR-210012':
			$msg = __('Inserted wrong PIN code in banking application.', 'imoje');
			break;
		case 'BLK-ERROR-210013':
			$msg = __('Payment failed (security).', 'imoje');
			break;
		case 'BLK-ERROR-210014':
			$msg = __('Limit exceeded in your banking application.', 'imoje');
			break;
		case 'BLK-ERROR-210015':
			$msg = __('Insufficient funds in your bank account.', 'imoje');
			break;
		case 'BLK-ERROR-210016':
			$msg = __('Issuer declined.', 'imoje');
			break;
		case 'BLK-ERROR-210017':
			$msg = __('Transaction not found.', 'imoje');
			break;
		case 'BLK-ERROR-210018':
			$msg = __('Bad IBAN.', 'imoje');
			break;
		case 'BLK-ERROR-210019':
			$msg = __('Transfer not possible.', 'imoje');
			break;
		case 'BLK-ERROR-210020':
			$msg = __('Return late.', 'imoje');
			break;
		case 'BLK-ERROR-210021':
			$msg = __('Return amount exceeded.', 'imoje');
			break;
		case 'BLK-ERROR-210022':
			$msg = __('Transfer late.', 'imoje');
			break;
		default:
			return __('Payment failed.', 'imoje') . ' ' . $tEnterT6;
	}

	return $msg . ' ' . $tEnterT6;
}
// endregion
